#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
""" Tests for Locations Report """

# pylint: disable=E0239, E0401, E0402, W0611, W0621

from pathlib import Path
from typing import NamedTuple
from unittest.mock import MagicMock, patch

import marshmallow.exceptions
import pytest
import requests_mock
from productfetcher.interfaces import LocationReport
from productfetcher.locations import (
    Cluster,
    FileLocator,
    Location,
    NgasFile,
    NgasFileSchema,
    NgasServer,
    NgasServerSchema,
    OracleXml,
    ServiceLocator,
)


@pytest.fixture
def cal_json_file(resource_path_root) -> Path:
    """
    Get the test data location file for calibration.

    :return:
    """
    return resource_path_root / "location_files" / "CALIBRATION.json"


@pytest.fixture
def eb_17a_109_18468_locations_file(resource_path_root) -> Path:
    """
    Grab all the file things from one of our VLA test .jsons

    :return:
    """
    return resource_path_root / "location_files" / "17A-109_fg_18468.json"


@pytest.fixture
def alma_execblock_report(resource_path_root) -> Path:
    """
    Get our ALMA locations file.

    :return: locations file
    """
    return resource_path_root / "location_files" / "alma-execblock.json"


def test_loads_simple_locations_report(cal_json_file: Path):
    """
    Can marshmallow load an entire JSON locations report?

    :return:
    """
    report = FileLocator(cal_json_file).locate()
    assert isinstance(report, LocationReport)


def test_loads_complex_locations_report(eb_17a_109_18468_locations_file):
    """
    Can marshmallow load an entire JSON locations report?

    :return:
    """
    locations_rpt = FileLocator(eb_17a_109_18468_locations_file).locate()
    assert isinstance(locations_rpt, LocationReport)

    files = locations_rpt.files
    assert len(files) == 79
    assert locations_rpt.aggregate_size == 45572444823


def test_loads_alma_locations_report(alma_execblock_report):
    """
    Can marshmallow load an entire JSON locations report?

    :return:
    """
    locations_rpt = FileLocator(alma_execblock_report).locate()
    assert isinstance(locations_rpt, LocationReport)

    assert len(locations_rpt.files) == 1099
    assert len([file for file in locations_rpt.files if isinstance(file, OracleXml)]) == 36
    assert len([file for file in locations_rpt.files if isinstance(file, NgasFile)]) == 1063
    assert locations_rpt.aggregate_size == 4119667881


class FakeProductfetcherSettings(NamedTuple):
    """
    Represents Capo settings for product fetcher

    """

    defaultThreadsPerHost: int
    executionSite: str
    locatorServiceUrlPrefix: str


def test_loads_from_service(eb_17a_109_18468_locations_file: Path):
    """
    Can we access a web service and retrieve a location report?

    :param eb_17a_109_18468_locations_file: report file to use as a fake
    :return:
    """
    settings = FakeProductfetcherSettings(4, "DSOC", "http://localhost/location?locator=")
    with patch("productfetcher.locations.CapoConfig") as capo:
        capo.return_value = MagicMock()
        capo.return_value.settings.return_value = settings
        with requests_mock.Mocker() as mock:
            mock.get(
                "http://localhost/location?locator=pretend-uid",
                text=eb_17a_109_18468_locations_file.read_text(),
            )
            locations_rpt = ServiceLocator("pretend-uid").locate()
            assert len(locations_rpt.files) == 79
            assert locations_rpt.aggregate_size == 45572444823


def test_direct_copy_detection():
    """
    Ensure that we properly detect various scenarios when we can use
    the direct copy plugin.

    :return:
    """
    aoc = Path("/lustre/aoc/foo")
    naasc = Path("/lustre/naasc/foo")
    home = Path("/home/nobody/foo")

    # there are two cases we can direct copy to, currently: DSOC -> DSOC
    # and NAASC -> NAASC (with cluster=DSOC)
    assert NgasServer("", Location.DSOC, Cluster.DSOC).can_direct_copy_to(aoc)
    assert NgasServer("", Location.NAASC, Cluster.DSOC).can_direct_copy_to(naasc)

    # all the other permutations we cannot: NAASC -> AOC, AOC -> NAASC, NAASC -> NAASC
    assert not NgasServer("", Location.NAASC, Cluster.DSOC).can_direct_copy_to(aoc)
    assert not NgasServer("", Location.DSOC, Cluster.NAASC).can_direct_copy_to(aoc)
    assert not NgasServer("", Location.DSOC, Cluster.NAASC).can_direct_copy_to(naasc)
    assert not NgasServer("", Location.NAASC, Cluster.NAASC).can_direct_copy_to(naasc)
    assert not NgasServer("", Location.NAASC, Cluster.NAASC).can_direct_copy_to(aoc)

    # and of course, we can never direct copy to your house
    for location in Location:
        for cluster in Cluster:
            assert not NgasServer("", location, cluster).can_direct_copy_to(home)


class TestNgasFileSchema:
    def test_null_value_in_subdirectory_field(self):
        """
        Tests that when the subdirectory value is provided to NgasFileSchema it will accept it and fill it in with
        a default value
        """
        fake_oracle_xml_data = {
            "archive_uid": "1341983",
            "table": "cloth",
            "subdirectory": None,
            "relative_path": "relpath",
            "size": 1000000,
        }
        try:
            fake_file = NgasFileSchema().load(data=fake_oracle_xml_data)
            assert type(fake_file) is OracleXml
            assert fake_file.subdirectory == "retrieved-products"
        except marshmallow.exceptions.ValidationError:
            pytest.fail("Subdirectory field should be nullable.")

        fake_ngas_file_data = {
            "ngas_file_id": "12345",
            "subdirectory": None,
            "relative_path": "relpath",
            "checksum": 12345,
            "checksum_type": "foo",
            "version": 1,
            "size": 1000000,
            "server": {"server": "nmngas07", "location": "DSOC", "cluster": "DSOC"},
        }
        try:
            fake_file = NgasFileSchema().load(data=fake_ngas_file_data)
            assert type(fake_file) is NgasFile
            assert fake_file.subdirectory == "retrieved-products"
        except marshmallow.exceptions.ValidationError:
            pytest.fail("Subdirectory field should be nullable.")
