#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Extracts workflow relevant metadata from the NRAO archive based on Science Product Locator.
"""
import json
import logging
from typing import List

import pendulum
from aat_wrest.utilities import PENDULUM_FORMAT, TIME_ZONE, MDDBConnector


class WrestWorkflowMetadata:
    """
    Class for extracting metadata required for workflow execution
    """

    def __init__(
        self,
        connection: MDDBConnector,
        spl: List[str] = None,
        sdm_id: str = None,
    ):
        self.logger = logging.getLogger("aat_wrest")
        self.conn = connection
        self.sdm_id = sdm_id
        self.spl = spl

    def wrest_standard_cal_info(self) -> json:
        """
        Given an execution block science product locator, returns the required metadata to run
        the standard calibration workflow
        :return:
        """

        sql = f"""
        SELECT ngas_fileset_id as filesetId,
         e.project_code as projectCode,
         p.title as title,
         e.starttime as startTime,
         (a.firstname || ' ' || a.lastname) as observer,
         telescope as telescope
         FROM execution_blocks e
            JOIN projects p on e.project_code = p.project_code
            JOIN authors a on p.project_code = a.project_code
         WHERE science_product_locator = %(spl)s AND a.is_pi = true
        """
        make_json = {}
        try:
            cursor = self.conn.cursor()
            cursor.execute(sql, {"spl": self.spl})
            data = cursor.fetchall()
            if data:
                make_json = json.dumps(
                    {
                        "sdmId": data[0][0],
                        "projectCode": data[0][1],
                        "title": data[0][2],
                        "startTime": data[0][3],
                        "observer": data[0][4],
                        "telescope": data[0][5],
                        "created_at": str(pendulum.now().in_timezone(TIME_ZONE).format(PENDULUM_FORMAT)),
                    }
                )
            else:
                self.logger.error(
                    f"ERROR: aat-wrest query returned no results!"
                    f" The database appears to be missing information for SPL {self.spl}!"
                )
        finally:
            self.conn.close()
        return make_json

    def wrest_standard_image_info(self) -> json:
        """
        Given an execution block SDM ID, returns the required metadata to run
        the standard imaging workflow
        :return:
        """

        sql = f"""
        SELECT e.science_product_locator as spl,
         e.project_code as projectCode,
         p.title as title,
         e.starttime as startTime,
         (a.firstname || ' ' || a.lastname) as observer,
         telescope as telescope,
         c.science_product_locator as cal_locator
         FROM execution_blocks e
            JOIN projects p on e.project_code = p.project_code
            JOIN authors a on p.project_code = a.project_code
            LEFT JOIN calibrations c on e.execution_block_id = c.execution_block_id
         WHERE ngas_fileset_id = %(sdmId)s AND a.is_pi = true
        """
        make_json = {}
        try:
            cursor = self.conn.cursor()
            cursor.execute(sql, {"sdmId": self.sdm_id})
            data = cursor.fetchall()
            if data:
                make_json = json.dumps(
                    {
                        "product_locator": data[0][0],
                        "projectCode": data[0][1],
                        "title": data[0][2],
                        "startTime": data[0][3],
                        "observer": data[0][4],
                        "telescope": data[0][5],
                        "cal_locator": data[0][6],
                        "created_at": str(pendulum.now().in_timezone(TIME_ZONE).format(PENDULUM_FORMAT)),
                    }
                )
            else:
                self.logger.error(
                    f"ERROR: aat-wrest query returned no results!"
                    f" The database appears to be missing information for sdm id: {self.sdm_id}!"
                )
        finally:
            self.conn.close()
        return make_json

    def wrest_restore_info(self) -> json:
        """
        Given an execution block science product locator and calibration science product locator, returns
        the required metadata to run the restore CMS workflow
        :return:
        """
        eb_sql = f"""
        SELECT ngas_fileset_id as filesetId,
         e.project_code as projectCode,
         p.title as title,
         e.starttime as startTime,
         (a.firstname || ' ' || a.lastname) as observer,
         telescope as telescope
         FROM execution_blocks e
            JOIN projects p on e.project_code = p.project_code
            JOIN authors a on p.project_code = a.project_code
         WHERE science_product_locator = %(spl)s AND a.is_pi = true
        """

        cal_sql = f"""
        SELECT external_name as calSdmId
        from science_products WHERE science_product_locator = %(cal_spl)s
        """

        make_json = {}
        try:
            cursor = self.conn.cursor()
            cursor.execute(eb_sql, {"spl": self.spl[0]})
            data = cursor.fetchall()
            cursor.execute(cal_sql, {"cal_spl": self.spl[1]})
            cal_data = cursor.fetchall()
            if data and cal_data:
                make_json = json.dumps(
                    {
                        "sdmId": data[0][0],
                        "calSdmId": cal_data[0][0],
                        "projectCode": data[0][1],
                        "title": data[0][2],
                        "startTime": data[0][3],
                        "observer": data[0][4],
                        "telescope": data[0][5],
                        "created_at": str(pendulum.now().in_timezone(TIME_ZONE).format(PENDULUM_FORMAT)),
                    }
                )
            else:
                self.logger.error(
                    f"ERROR: aat-wrest query returned no results!"
                    f" The database appears to be missing information for either SPL {self.spl[0]} or Cal SPL {self.spl[1]}!"
                )
        finally:
            self.conn.close()
        return make_json

    def wrest_aux_imaging_info(self) -> json:
        """
        Given a parent workflow request id for an image ingestion, returns the required metadata to run
        the image ingestion
        :return:
        """

        if self.spl and "calibration" in self.spl:
            sql = """
            SELECT e.project_code as projectCode,
                band_code as bandCodes,
                configuration
            FROM execution_blocks e
            JOIN calibrations c on e.execution_block_id = c.execution_block_id
            """

            condition = f"WHERE c.science_product_locator = '{self.spl}'"

        else:
            sql = """
            SELECT project_code as projectCode,
                band_code as bandCodes,
                configuration
            FROM execution_blocks e
            """

            condition = (
                f"WHERE science_product_locator = '{self.spl}'"
                if self.spl is not None
                else f"WHERE ngas_fileset_id = '{self.sdm_id}'"
            )

        make_json = {}
        try:
            cursor = self.conn.cursor()
            cursor.execute(sql + condition)
            data = cursor.fetchall()
            if data:
                make_json = json.dumps(
                    {
                        "projectCode": data[0][0],
                        "bands": data[0][1],
                        "configurations": data[0][2],
                    }
                )
            else:
                self.logger.error(
                    f"ERROR: aat-wrest query returned no results!"
                    f" The database appears to be missing information for sdm id {self.sdm_id} or spl id {self.spl}!"
                )
        finally:
            self.conn.close()
        return make_json

    def wrest_image_name(self) -> json:
        """
        Given an image SPL, returns the image file name
        :return:
        """

        sql = """
        SELECT external_name as image_name
        FROM science_products
        WHERE science_product_locator = %(spl)s
            """

        make_json = {}
        try:
            cursor = self.conn.cursor()
            cursor.execute(sql, {"spl": self.spl[0]})
            data = cursor.fetchall()
            if data:
                make_json = json.dumps(
                    {
                        "imageName": data[0][0],
                    }
                )
            else:
                self.logger.error(
                    f"ERROR: aat-wrest query returned no results!"
                    f" The database appears to be missing information for spl id {self.spl[0]}!"
                )
        finally:
            self.conn.close()
        return make_json

    def wrest_group_ancillary_locator(self, ancillary_type: str = "pipeline_weblog") -> json:
        """
        Given the partial external name of a product, retrieve the locator for a specific
        ancillary product related to that product.

        :param ancillary_type:
        :return: A JSON blob with the locator
        """

        sql = """
        SELECT ancillary_product_locator
        FROM ancillary_products
        JOIN science_products_product_groups sppg on ancillary_products.product_group_id = sppg.product_group_id
        JOIN science_products sp on sppg.science_product_locator = sp.science_product_locator
        WHERE sp.external_name LIKE %(partial_name)s
        AND ancillary_product_type = %(ancillary_type)s
        """

        make_json = {}
        try:
            cursor = self.conn.cursor()
            name_like = self.sdm_id + "%"
            self.logger.debug(name_like)
            self.logger.debug(ancillary_type)
            cursor.execute(sql, {"partial_name": name_like, "ancillary_type": ancillary_type})
            data = cursor.fetchall()
            if data:
                make_json = json.dumps({"product_locator": data[0][0]})
            else:
                self.logger.error(
                    f"ERROR: aat-wrest query returned no results!"
                    f" The database appears to be missing information for sdm id: {self.sdm_id}!"
                )
        finally:
            self.conn.close()
        return make_json
